/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_TIME "pygame module for monitoring time"
#define DOC_TIME_GETTICKS "get_ticks() -> milliseconds\nget the time in milliseconds"
#define DOC_TIME_WAIT "wait(milliseconds, /) -> time\npause the program for an amount of time"
#define DOC_TIME_DELAY "delay(milliseconds, /) -> time\npause the program for an amount of time"
#define DOC_TIME_SETTIMER "set_timer(event, millis) -> None\nset_timer(event, millis, loops=0) -> None\nrepeatedly create an event on the event queue"
#define DOC_TIME_CLOCK "Clock() -> Clock\ncreate an object to help track time"
#define DOC_TIME_CLOCK_TICK "tick(framerate=0, /) -> milliseconds\nupdate the clock"
#define DOC_TIME_CLOCK_TICKBUSYLOOP "tick_busy_loop(framerate=0, /) -> milliseconds\nupdate the clock"
#define DOC_TIME_CLOCK_GETTIME "get_time() -> milliseconds\ntime used in the previous tick"
#define DOC_TIME_CLOCK_GETRAWTIME "get_rawtime() -> milliseconds\nactual time used in the previous tick"
#define DOC_TIME_CLOCK_GETFPS "get_fps() -> float\ncompute the clock framerate"
