/* Auto generated file: with make_docs.py .  Docs go in docs/reST/ref/ . */
#define DOC_CURSORS "pygame module for cursor resources"
#define DOC_CURSORS_COMPILE "compile(strings, black='X', white='.', xor='o') -> data, mask\ncreate binary cursor data from simple strings"
#define DOC_CURSORS_LOADXBM "load_xbm(cursorfile) -> cursor_args\nload_xbm(cursorfile, maskfile) -> cursor_args\nload cursor data from an XBM file"
#define DOC_CURSORS_CURSOR "Cursor(size, hotspot, xormasks, andmasks) -> Cursor\nCursor(hotspot, surface) -> Cursor\nCursor(constant) -> Cursor\nCursor(Cursor) -> Cursor\nCursor() -> Cursor\npygame object representing a cursor"
#define DOC_CURSORS_CURSOR_COPY "copy() -> Cursor\ncopy the current cursor"
#define DOC_CURSORS_CURSOR_TYPE "type -> string\nget the cursor type"
#define DOC_CURSORS_CURSOR_DATA "data -> tuple\nget the cursor data"
